@@ -14,7 +14,7 @@ from jf_ingest.constants import Constants
 from jf_ingest.jf_git.exceptions import GitProviderUnavailable
 from jf_ingest.jf_git.standardized_models import StandardizedRepository
 from jf_ingest.logging_helper import send_to_agent_log_file
-from jf_ingest.utils import format_datetime_to_ingest_timestamp
+from jf_ingest.utils import format_datetime_to_ingest_timestamp, normalize_datetime
 
 logger = logging.getLogger(__name__)
 
@@ -310,6 +310,7 @@ class GitConfig:
     git_redact_names_and_urls: bool = False
     git_strip_text_content: bool = False
     check_ghc_mannequin_user_prs: bool = False
+    include_pr_comment_reactions: bool = True
 
     def __post_init__(self):
         if type(self.git_provider) == str:
@@ -446,6 +447,9 @@ class JiraDownloadConfig(JiraAuthConfig):
     include_project_categories: List = field(default_factory=list)
     exclude_project_categories: List = field(default_factory=list)
 
+    # Components
+    download_global_components: bool = False
+
     # Boards/Sprints
     download_boards: bool = True
     download_sprints: bool = True
@@ -498,26 +502,18 @@ class JiraDownloadConfig(JiraAuthConfig):
             )
             return
 
-        def normalize_datetime(dt):
-            if type(dt) is datetime:
-                return dt.astimezone(tz)
-            else:
-                # Sometimes objects are passed in as Dates and not Datetimes,
-                # so we can't adjust the timezone on them
-                return dt
-
-        self.pull_from = normalize_datetime(self.pull_from)
-        self.work_logs_pull_from = normalize_datetime(self.work_logs_pull_from)
+        self.pull_from = normalize_datetime(self.pull_from, tz)
+        self.work_logs_pull_from = normalize_datetime(self.work_logs_pull_from, tz)
 
         # Normalize the project_id_to_pull_from datetimes
         if self.project_id_to_pull_from:
             for project_id, pull_date in self.project_id_to_pull_from.items():
-                self.project_id_to_pull_from[project_id] = normalize_datetime(pull_date)
+                self.project_id_to_pull_from[project_id] = normalize_datetime(pull_date, tz)
 
         # Normalize issue metadata datetimes
         for issue in self.jellyfish_issue_metadata:
             if issue.updated:
-                issue.updated = normalize_datetime(issue.updated)
+                issue.updated = normalize_datetime(issue.updated, tz)
 
         send_to_agent_log_file(
             f'Successfully normalized datetimes to timezone locale: {timezone_locale}',
