@@ -12,6 +12,7 @@ logger = logging.getLogger(__name__)
 
 
 GQL_PAGE_INFO_BLOCK = "pageInfo {hasNextPage, endCursor}"
+GQL_RATELIMIT_INFO_BLOCK = "rateLimit {cost, remaining, resetAt}"
 
 
 @dataclass
@@ -35,3 +36,7 @@ def gql_format_to_datetime(datetime_str: str) -> datetime:
         datetime: A valid, timezone aware datetime
     """
     return datetime.strptime(datetime_str, "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
+
+
+def _gql_query_has_next_page(objects_query: dict) -> bool:
+    return bool(objects_query.get('pageInfo', {}).get('hasNextPage', False))
