@@ -113,6 +113,10 @@ class GitAdapter(ABC):
         30000  # Commits are generally uniform in size. This is ~50 MBs per commit batch
     )
 
+    @staticmethod
+    def generate_unknown_emoji_code(emoji_code: str) -> str:
+        return f'UNKNOWN ({emoji_code})'
+
     def _transform_data_objects_before_saving(
         self,
         dataclass_objects: (
@@ -763,30 +767,6 @@ class GitAdapter(ABC):
                 f'Not pulling users because \'skip_pulling_users\' is set to: {git_config.skip_pulling_users}.'
             )
 
-        #######################################################################
-        # TEAM DATA
-        #######################################################################
-        with record_span('get_teams'):
-            logger.info('Fetching Git Team Data...')
-            standardized_teams: List[StandardizedTeam] = (
-                [
-                    team
-                    for org in standardized_organizations
-                    for team in tqdm_to_logger(
-                        self.get_teams(org), desc="Processing Teams", unit=" teams"
-                    )
-                ]
-                if self.config.pull_teams
-                else []
-            )
-            logger.info(f'Successfully found {len(standardized_teams)} teams.')
-            add_telemetry_fields({'git_team_count': len(standardized_teams)})
-        # Upload Data
-        _write_to_s3_or_local(
-            object_name=GitObject.GitTeams.value,
-            json_data=self._transform_data_objects_before_saving(standardized_teams),
-        )
-
         #######################################################################
         # REPO DATA, NOTE THAT WE UPLOAD LATER BECAUSE WE NEED TO SET
         # THE BACKPOPULATD DATES BELOW AFTER WE PULL PRS AND COMMITS
