@@ -8,7 +8,7 @@ from dateutil import parser
 from requests.exceptions import HTTPError
 from tqdm import tqdm
 
-from jf_ingest import diagnostics, logging_helper
+from jf_ingest import logging_helper
 from jf_ingest.config import GitConfig
 from jf_ingest.graphql_utils import gql_format_to_datetime
 from jf_ingest.jf_git.adapters import GitAdapter
@@ -24,6 +24,7 @@ from jf_ingest.jf_git.standardized_models import (
     StandardizedPullRequest,
     StandardizedPullRequestAuthor,
     StandardizedPullRequestComment,
+    StandardizedPullRequestCommentReaction,
     StandardizedPullRequestMetadata,
     StandardizedPullRequestReview,
     StandardizedPullRequestReviewAuthor,
@@ -69,7 +70,9 @@ class GithubRepoMetaData(NamedTuple):
 class GithubAdapter(GitAdapter):
     def __init__(self, config: GitConfig):
         # Git Config options
-        self.client = GithubClient(config.git_auth_config)
+        self.client = GithubClient(
+            config.git_auth_config, include_reactions=config.include_pr_comment_reactions
+        )
         self.config = config
         self.repo_id_to_name_lookup: Dict = {}
         self.repo_id_to_repo_meta_data: Dict[str, GithubRepoMetaData] = {}
@@ -77,6 +80,23 @@ class GithubAdapter(GitAdapter):
     def get_api_scopes(self):
         return self.client.get_scopes_of_api_token()
 
+    @staticmethod
+    def get_unicode_from_reaction_content(reaction_content: str) -> str:
+        reaction_content.upper()
+        reaction_mapping = {
+            "THUMBS_UP": "U+1F44D",  # Thumbs Up
+            "THUMBS_DOWN": "U+1F44E",  # Thumbs Down
+            "LAUGH": "U+1F606",  # Laugh
+            "HOORAY": "U+1F389",  # Hooray
+            "CONFUSED": "U+1F615",  # Confused
+            "HEART": "U+2764",  # Heart
+            "ROCKET": "U+1F680",  # Rocket
+            "EYES": "U+1F440",  # Eyes
+        }
+        return reaction_mapping.get(
+            reaction_content, GitAdapter.generate_unknown_emoji_code(reaction_content)
+        )
+
     def get_organizations(self) -> List[StandardizedOrganization]:
         # NOTE: For github, organization equates to a Github organization here!
         # We have the name of the organization in the config, expected to be a list of length one
@@ -137,7 +157,11 @@ class GithubAdapter(GitAdapter):
             # NOTE: Default branch can be None!
             latest_commit_date_on_default_branch: Optional[datetime] = None
             if default_branch := api_repo.get('defaultBranch'):
-                if commits := default_branch['commitsQuery']['commitHistory']['commits']:
+                if (
+                    commits := default_branch.get('commitsQuery', {})
+                    .get('commitHistory', {})
+                    .get('commits')
+                ):
                     latest_commit_date_on_default_branch = gql_format_to_datetime(
                         commits[0]['committedDate']
                     )
@@ -567,6 +591,27 @@ class GithubAdapter(GitAdapter):
             )
             logging_helper.send_to_agent_log_file(traceback.format_exc(), level=logging.ERROR)
 
+    def get_comment_and_reactions_for_prs(
+        self, login: str, repo_name: str, pr_numbers: list[int]
+    ) -> Generator[tuple[int, list[StandardizedPullRequestComment]], None, None]:
+        """This is a function that is specific, currently, only to Github.
+        It is responsible for making effecient API calls to fetch reviews,
+        comments, and reactions, for a set of PRs. This is the helper
+        function for the "historic backill" for reaction data
+
+        Args:
+            login (str): A login name (the organization slug)
+            repo_name (str): The repo name we're concerned with
+            pr_numbers (list[int]): A list of PR numbers to fetch historical data for
+
+        Yields:
+            Generator[tuple[int, list[StandardizedPullRequestComment]], None, None]: Yield a pr number and a list of standardized comments
+        """
+        for pr_number, api_comments in self.client.get_pr_comments_and_reactions(
+            login, repo_name, pr_numbers
+        ):
+            yield pr_number, _get_standardized_pr_comments(api_comments)
+
 
 '''
 
@@ -694,11 +739,27 @@ def _get_standardized_pr_comments(api_comments: List[Dict]) -> List[Standardized
             user=_standardize_user(api_comment['author']),
             body=api_comment['body'],
             created_at=parser.parse(api_comment['createdAt']),
+            reactions=_get_standardized_reactions(
+                api_comment.get('reactionsQuery', {}).get('reactions', [])
+            ),
         )
         for api_comment in api_comments
     ]
 
 
+def _get_standardized_reactions(
+    api_reactions: List[Dict],
+) -> List[StandardizedPullRequestCommentReaction]:
+    return [
+        StandardizedPullRequestCommentReaction(
+            user=_standardize_user(api_reaction['user']),
+            emoji_code=GithubAdapter.get_unicode_from_reaction_content(api_reaction['content']),
+            created_at=parser.parse(api_reaction['createdAt']),
+        )
+        for api_reaction in api_reactions
+    ]
+
+
 def _get_standardized_reviews(api_reviews: List[Dict]):
     return [
         StandardizedPullRequestReview(
