@@ -14,8 +14,10 @@ from jf_ingest.config import GitAuthConfig, GithubAuthConfig
 from jf_ingest.constants import Constants
 from jf_ingest.graphql_utils import (
     GQL_PAGE_INFO_BLOCK,
+    GQL_RATELIMIT_INFO_BLOCK,
     GQLRateLimit,
     GqlRateLimitedExceptionInner,
+    _gql_query_has_next_page,
     gql_format_to_datetime,
 )
 from jf_ingest.jf_git.exceptions import (
@@ -34,8 +36,6 @@ logger = logging.getLogger(__name__)
 
 GIT_DEFAULT_API_ENDPOINT = 'https://api.github.com'
 
-GQL_RATE_LIMIT_QUERY_BLOCK = "rateLimit {remaining, resetAt}"
-
 GITHUB_STATUSES_TO_RETRY_ON = tuple(list(DEFAULT_HTTP_CODES_TO_RETRY_ON) + [403])
 
 
@@ -85,6 +85,7 @@ class GithubClient:
     def __init__(
         self,
         github_auth_config: GitAuthConfig,
+        include_reactions: bool = True,
         **kwargs,
     ):
         """This is a wrapper class for interacting with the Github GQL endpoint. It supports
@@ -92,14 +93,8 @@ class GithubClient:
         the other args to set up JWT authentication (see: https://developer.github.com/apps/building-github-apps/authenticating-with-github-apps/)
 
         Args:
-            token (str, optional): An optional authentication token. If not provided we will use the other args to get a JWT token. See GithubClient._get_app_access_token. Defaults to None.
-            installation_id (str, optional): A client provided installation ID, used for JWT authentication. Defaults to None.
-            app_id (str, optional): A client provided App Id, used for JWT authentication. Defaults to None.
-            private_key (str, optional): A user provided private key, used for JWT authentication. Defaults to None.
-            base_url (str, optional): An optional base url, used by Github Enterprise customers. Defaults to None.
-            verify (bool, optional): Session arg. Defaults to True.
-            session (requests.Session, optional): An optional session to pass this client, if you've already set up an authenticated session. Defaults to None.
-            company_slug (str, optional): An optional str giving this class context what company this client is for. Defaults to None.
+            GitAuthConfig (GithubAuthConfig): A fully fleshed out GithubAuthConfig, with token, installation_id, app_id and private_key set (depending on GH or GHE), as well as base_url, verify, and company_slug
+            include_reactions
         """
 
         self.company_slug = github_auth_config.company_slug
@@ -107,6 +102,8 @@ class GithubClient:
         # We need to hit the REST API for some API calls, see get_organization_by_name and _get_app_access_token
         self.rest_api_url = github_auth_config.base_url or GIT_DEFAULT_API_ENDPOINT
 
+        self.include_reactions = include_reactions
+
         # Create the user fragment based on the provider being GHE or GHC
         if (
             isinstance(github_auth_config.base_url, str)
@@ -202,7 +199,7 @@ class GithubClient:
         Returns:
             dict: A dictionary object containing rate limit information (remaing and resetAt)
         """
-        query_body = f"{{{GQL_RATE_LIMIT_QUERY_BLOCK}}}"
+        query_body = f"{{{GQL_RATELIMIT_INFO_BLOCK}}}"
         # NOTE: DO NOT CALL get_raw_gql_result TO GET THE RESULTS HERE! IT'S A RECURSIVE TRAP
         response: requests.Response = retry_for_status(
             self.session.post, url=self.gql_base_url, json={'query': query_body}
@@ -213,7 +210,7 @@ class GithubClient:
         reset_at = cast(
             datetime, gql_format_to_datetime(raw_data['resetAt'])
         )  # resetAt is always going to be present
-        logger.info('GQL rate limit info', extra=raw_data)
+        logger.info(f'GQL rate limit info: {json.dumps(raw_data)}', extra=raw_data)
         return GQLRateLimit(remaining=int(raw_data['remaining']), reset_at=reset_at)
 
     def get_raw_result(self, query_body: str, max_attempts: int = 7) -> Dict:
@@ -242,14 +239,18 @@ class GithubClient:
                 response.raise_for_status()
                 json_str = response.content.decode()
                 json_data: Dict = json.loads(json_str)
+                if rate_limit_info_block := json_data.get('data', {}).get('rateLimit', {}):
+                    logger.debug(
+                        f"GQL Query Cost: {rate_limit_info_block.get('cost')} (remaining: {rate_limit_info_block.get('remaining')})"
+                    )
                 if 'errors' in json_data:
                     error_list: List[Dict] = json_data['errors']
                     if len(error_list) == 1:
                         error_dict = error_list[0]
                         error_type = error_dict.get('type', '')
-                        if error_type == 'RATE_LIMITED':
+                        if error_type in ('RATE_LIMITED', 'RATE_LIMIT'):
                             logger.info(
-                                'Rate limit hit in GQL, dumping headers',
+                                f'Rate limit hit in GQL, dumping headers.',
                                 extra={
                                     'x-ratelimit-limit': response.headers.get('x-ratelimit-limit'),
                                     'x-ratelimit-remaining': response.headers.get(
@@ -328,9 +329,9 @@ class GithubClient:
                 curr_timestamp = datetime.now(tz=timezone.utc).timestamp()
 
                 # Sometimes there is a race condition where we get here and our rate limiter has already reset.
-                # if that's the case, don't sleep. If, however, our remaining budget is 0 then we're going to
-                # have to sleep until it resets
-                if rate_limit_info.remaining == 0:
+                # if that's the case, don't sleep. If, however, our remaining budget is less than 100 or so
+                # then we're going to have to sleep until it resets
+                if rate_limit_info.remaining < 100:
                     # Convert float values to int, add one second as a grace period
                     sleep_time = int(reset_at_timestamp - curr_timestamp) + 1
 
@@ -578,6 +579,7 @@ class GithubClient:
 
     def get_users(self, login: str) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: \"{login}\") {{
                 userQuery: membersWithRole(first: 100, after: %s) {{
                     {GQL_PAGE_INFO_BLOCK}
@@ -596,6 +598,7 @@ class GithubClient:
 
     def get_team_members(self, login: str, team_slug: str) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: \"{login}\") {{
                 team(slug: \"{team_slug}\") {{
                     membersQuery: members(first: 100, after: %s) {{
@@ -616,6 +619,7 @@ class GithubClient:
 
     def get_teams(self, login: str) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: \"{login}\") {{
                 teamsQuery: teams(first: 100, after: %s) {{
                     {GQL_PAGE_INFO_BLOCK}
@@ -658,6 +662,7 @@ class GithubClient:
         priv_filter = 'privacy: PRIVATE, ' if only_private else ''
 
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repoQuery: repositories({priv_filter}first: 50, after: %s) {{
                     {GQL_PAGE_INFO_BLOCK}
@@ -692,6 +697,7 @@ class GithubClient:
 
     def get_branches(self, login: str, repo_name: str) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repository(name: "{repo_name}") {{
                     ... on Repository {{
@@ -719,6 +725,7 @@ class GithubClient:
         self, login: str, repo_name: str, branch_name: str, since: datetime
     ) -> int:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     ... on Repository {{
@@ -754,6 +761,7 @@ class GithubClient:
             f' until: "{self.datetime_to_gql_str_format(until)}",' if until else ""
         )
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     ... on Repository {{
@@ -787,25 +795,47 @@ class GithubClient:
     #
     # PR Queries are HUGE, so pull out reusable blocks (comments, reviews, commits, etc)
     #
-    def _get_pr_comments_query_block(self, enable_paging: bool = False):
+    def _get_pr_comments_query_block(
+        self, page_size: int, reactions_page_size: int, enable_paging: bool = False
+    ):
         return f"""
-            commentsQuery: comments(first: 100{', after: %s' if enable_paging else ''}) {{
+            commentsQuery: comments(first: {page_size}{', after: %s' if enable_paging else ''}) {{
                 {GQL_PAGE_INFO_BLOCK}
-
                 comments: nodes {{
                     author {{
                         {self.github_gql_actor_fragment}
                     }}
                     body
                     createdAt
+                    {self._get_reactions_query_block(page_size=reactions_page_size, enable_paging=False) if self.include_reactions else ''}
+                }}
+            }}
+        """
+
+    def _get_reactions_query_block(self, page_size: int, enable_paging: bool = False):
+        return f"""
+            reactionsQuery: reactions (first: {page_size}{', after: %s' if enable_paging else ''}) {{
+                pageInfo {{hasNextPage, endCursor}}
+                reactions: nodes {{
+                    user {{
+                        {self.github_gql_actor_fragment}
+                    }}
+                    content
+                    createdAt
                 }}
             }}
         """
 
     # NOTE: There are comments associated with reviews that we need to fetch as well
-    def _get_pr_reviews_query_block(self, enable_paging: bool = False):
+    def _get_pr_reviews_query_block(
+        self,
+        page_size: int,
+        comment_page_size: int,
+        reactions_page_size: int,
+        enable_paging: bool = False,
+    ):
         return f"""
-            reviewsQuery: reviews(first: 25{', after: %s' if enable_paging else ''}) {{
+            reviewsQuery: reviews(first: {page_size}{', after: %s' if enable_paging else ''}) {{
                 {GQL_PAGE_INFO_BLOCK}
 
                 reviews: nodes {{
@@ -815,16 +845,15 @@ class GithubClient:
                         }}
                         id: databaseId
                         state
-                        # NOTE! We are paging for comments here as well!
-                        {self._get_pr_comments_query_block()}
+                        {self._get_pr_comments_query_block(page_size=comment_page_size, reactions_page_size=reactions_page_size,  enable_paging=False)}
                     }}
                 }}
             }}
         """
 
-    def _get_labels_query_block(self, enable_paging: bool = False):
+    def _get_labels_query_block(self, page_size: int, enable_paging: bool = False):
         return f"""
-            labelsQuery: labels(first: 50{', after: %s' if enable_paging else ''}) {{
+            labelsQuery: labels(first: {page_size}{', after: %s' if enable_paging else ''}) {{
                 {GQL_PAGE_INFO_BLOCK}
                 labels: nodes {{
                     ... on Label {{
@@ -837,9 +866,9 @@ class GithubClient:
             }}
         """
 
-    def _get_pr_files_query_block(self, enable_paging: bool = False):
+    def _get_pr_files_query_block(self, page_size: int, enable_paging: bool = False):
         return f"""
-            filesQuery: files(first: 50{', after: %s' if enable_paging else ''}) {{
+            filesQuery: files(first: {page_size}{', after: %s' if enable_paging else ''}) {{
                 {GQL_PAGE_INFO_BLOCK}
                 files: nodes {{
                     ... on PullRequestChangedFile {{
@@ -853,12 +882,12 @@ class GithubClient:
         """
 
     def _get_pr_commits_query_block(
-        self, enable_paging: bool = False, include_force_push_commits: bool = False
+        self, page_size: int, enable_paging: bool = False, include_force_push_commits: bool = False
     ):
         # Force push commits use a different gql element
         if include_force_push_commits:
             return f"""
-                commitsQuery: timelineItems(first: 50{', after: %s' if enable_paging else ''}) {{
+                commitsQuery: timelineItems(first: {page_size}{', after: %s' if enable_paging else ''}) {{
                     {GQL_PAGE_INFO_BLOCK}
 
                     commits: nodes {{
@@ -892,6 +921,7 @@ class GithubClient:
 
     def get_prs_count(self, login: str, repo_name: str) -> int:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     prQuery: pullRequests(first: 1, orderBy: {{direction: DESC, field: UPDATED_AT}}) {{
@@ -922,6 +952,7 @@ class GithubClient:
         }
         """
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     prQuery: pullRequests(first: 100, orderBy: {{direction: DESC, field: UPDATED_AT}}, after: %s) {{
@@ -1064,7 +1095,7 @@ class GithubClient:
                 f"""
                         pr_{n}: pullRequest(number: {n}) {{
                             number
-                            reviews(first: 50) {{
+                            reviews(first: 25) {{
                                 nodes {{
                                     ... on PullRequestReview {{
                                         id: databaseId
@@ -1165,22 +1196,46 @@ class GithubClient:
 
         return pr_review_authors
 
-    # PR query is HUGE, see above GITHUB_GQL_PR_* blocks for reused code
-    # page_size is optimally variable. Most repos only have a 0 to a few PRs day to day,
-    # so sometimes the optimal page_size is 0. Generally, we should never go over 25
-    def get_prs(
+    def _need_to_fetch_with_deeper_reviews_query(self, api_pr: dict) -> bool:
+        if _gql_query_has_next_page(api_pr['reviewsQuery']):
+            logger.info(f'Need to query more reviews for PR {api_pr["number"]}')
+            return True
+        for review in api_pr['reviewsQuery']['reviews']:
+            if _gql_query_has_next_page(review['commentsQuery']):
+                logger.info(f'Need to query more comments for PR {api_pr["number"]}')
+                return True
+            for comment in review['commentsQuery']['comments']:
+                if _gql_query_has_next_page(comment.get('reactionsQuery', {})):
+                    logger.info(f'Need to query more reactions for PR {api_pr["number"]}')
+                    return True
+        return False
+
+    def get_pr_comments_from_reviews(self, reviews: list[dict]) -> list:
+        return [comment for review in reviews for comment in review['commentsQuery']['comments']]
+
+    def get_top_level_comments(
+        self, api_pr: dict, login: str, repo_name: str, pr_number: int
+    ) -> list:
+        return (
+            [
+                comment
+                for comment in self.get_pr_top_level_comments(login, repo_name, pr_number=pr_number)
+            ]
+            if api_pr['commentsQuery']['pageInfo']['hasNextPage']
+            else list(api_pr['commentsQuery']['comments'])
+        )
+
+    def _get_pr_query(
         self,
+        page_size: int,
         login: str,
         repo_name: str,
-        include_top_level_comments: bool = False,
-        include_force_push_commits: bool = False,
-        pull_files_for_pr: bool = False,
-        hash_files_for_prs: bool = False,
-        repository_label_node_ids_to_id: Dict[str, int] = {},
-        page_size: int = MAX_PAGE_SIZE_FOR_PR_QUERY,
-        start_cursor: Optional[Any] = None,
-    ) -> Generator[dict, None, None]:
-        query_body = f"""{{
+        include_force_push_commits: bool,
+        pull_files_for_pr: bool,
+        include_top_level_comments: bool,
+    ) -> str:
+        return f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     prQuery: pullRequests(first: {page_size}, orderBy: {{direction: DESC, field: UPDATED_AT}}, after: %s) {{
@@ -1214,11 +1269,11 @@ class GithubClient:
                                 mergeCommit {{
                                     {self.GITHUB_GQL_COMMIT_FRAGMENT}
                                 }}
-                                {self._get_pr_comments_query_block(enable_paging=False) if include_top_level_comments else ''}
-                                {self._get_pr_reviews_query_block(enable_paging=False)}
-                                {self._get_pr_commits_query_block(enable_paging=False, include_force_push_commits=include_force_push_commits)}
-                                {self._get_pr_files_query_block(enable_paging=False) if pull_files_for_pr else ''}
-                                {self._get_labels_query_block(enable_paging=False)}
+                                {self._get_pr_comments_query_block(page_size=10, reactions_page_size=5,  enable_paging=False) if include_top_level_comments else ''}
+                                {self._get_pr_reviews_query_block(page_size=20, comment_page_size=10, reactions_page_size=5,  enable_paging=False)}
+                                {self._get_pr_commits_query_block(page_size=100, enable_paging=False, include_force_push_commits=include_force_push_commits)}
+                                {self._get_pr_files_query_block(page_size=100, enable_paging=False) if pull_files_for_pr else ''}
+                                {self._get_labels_query_block(page_size=50, enable_paging=False)}
                             }}
                         }}
                     }}
@@ -1226,7 +1281,32 @@ class GithubClient:
             }}
         }}
         """
+
+    # PR query is HUGE, see above GITHUB_GQL_PR_* blocks for reused code
+    # page_size is optimally variable. Most repos only have a 0 to a few PRs day to day,
+    # so sometimes the optimal page_size is 0. Generally, we should never go over 25
+    def get_prs(
+        self,
+        login: str,
+        repo_name: str,
+        include_top_level_comments: bool = False,
+        include_force_push_commits: bool = False,
+        pull_files_for_pr: bool = False,
+        hash_files_for_prs: bool = False,
+        repository_label_node_ids_to_id: Dict[str, int] = {},
+        page_size: int = MAX_PAGE_SIZE_FOR_PR_QUERY,
+        start_cursor: Optional[Any] = None,
+    ) -> Generator[dict, None, None]:
         transformed_start_cursor = f'"{start_cursor}"' if start_cursor else None
+        pr_page_size = page_size if not self.include_reactions else min(5, page_size)
+        query_body = self._get_pr_query(
+            page_size=pr_page_size,
+            login=login,
+            repo_name=repo_name,
+            include_force_push_commits=include_force_push_commits,
+            pull_files_for_pr=pull_files_for_pr,
+            include_top_level_comments=include_top_level_comments,
+        )
         for page in self.page_results_gql(
             query_body=query_body,
             path_to_page_info='data.organization.repo.prQuery',
@@ -1238,9 +1318,9 @@ class GithubClient:
                 pr_number = api_pr['number']
 
                 # Load reviews first because we use them in both reviews and comments
-                reviews = (
+                api_pr['reviews'] = (
                     [r for r in self.get_pr_reviews(login, repo_name, pr_number=pr_number)]
-                    if api_pr['reviewsQuery']['pageInfo']['hasNextPage']
+                    if self._need_to_fetch_with_deeper_reviews_query(api_pr)
                     else api_pr['reviewsQuery']['reviews']
                 )
 
@@ -1249,27 +1329,17 @@ class GithubClient:
                 # but there are also comments associated with each review (typically only one)
                 # The baseline for what we care about is the Review Level comment, pulled from
                 # the reviews endpoint. Grabbing Top Level Comments is an optional feature flag
-
                 # Grab the comments pulled from reviews. We ALWAYS want these!
-                api_pr['comments'] = [
-                    comment for review in reviews for comment in review['commentsQuery']['comments']
-                ]
+                comments = []
+                comments.extend(self.get_pr_comments_from_reviews(api_pr['reviews']))
 
-                # Grab the potentially optional top level comments
                 if include_top_level_comments:
-                    top_level_comments = (
-                        [
-                            comment
-                            for comment in self.get_pr_top_level_comments(
-                                login, repo_name, pr_number=pr_number
-                            )
-                        ]
-                        if api_pr['commentsQuery']['pageInfo']['hasNextPage']
-                        else api_pr['commentsQuery']['comments']
+                    # If we need to page for top level comments, do so
+                    comments.extend(
+                        self.get_top_level_comments(api_pr, login, repo_name, pr_number)
                     )
-                    api_pr['comments'].extend(top_level_comments)
 
-                api_pr['reviews'] = reviews
+                api_pr['comments'] = comments
 
                 api_pr['commits'] = (
                     [
@@ -1338,11 +1408,12 @@ class GithubClient:
         self, login: str, repo_name: str, pr_number: int
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     pr: pullRequest(number: {pr_number}) {{
                         ... on PullRequest {{
-                            {self._get_pr_comments_query_block(enable_paging=True)}
+                            {self._get_pr_comments_query_block(page_size=100, reactions_page_size=100,  enable_paging=True)}
                         }}
                     }}
                 }}
@@ -1358,14 +1429,18 @@ class GithubClient:
                 yield api_pr_comment
 
     def get_pr_reviews(
-        self, login: str, repo_name: str, pr_number: int
+        self,
+        login: str,
+        repo_name: str,
+        pr_number: int,
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     pr: pullRequest(number: {pr_number}) {{
                         ... on PullRequest {{
-                            {self._get_pr_reviews_query_block(enable_paging=True)}
+                            {self._get_pr_reviews_query_block(page_size=25, comment_page_size=100, reactions_page_size=100,  enable_paging=True)}
                         }}
                     }}
                 }}
@@ -1384,11 +1459,12 @@ class GithubClient:
         self, login: str, repo_name: str, pr_number: int, include_force_push_commits: bool = False
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     pr: pullRequest(number: {pr_number}) {{
                         ... on PullRequest {{
-                            {self._get_pr_commits_query_block(enable_paging=True, include_force_push_commits=include_force_push_commits)}
+                            {self._get_pr_commits_query_block(page_size=100, enable_paging=True, include_force_push_commits=include_force_push_commits)}
                         }}
                     }}
                 }}
@@ -1420,11 +1496,12 @@ class GithubClient:
         self, login: str, repo_name: str, pr_number: int
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     pr: pullRequest(number: {pr_number}) {{
                         ... on PullRequest {{
-                            {self._get_pr_files_query_block(enable_paging=True)}
+                            {self._get_pr_files_query_block(page_size=100, enable_paging=True)}
                         }}
                     }}
                 }}
@@ -1441,11 +1518,12 @@ class GithubClient:
         self, login: str, repo_name: str, pr_number: int
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                 repo: repository(name: "{repo_name}") {{
                     pr: pullRequest(number: {pr_number}) {{
                         ... on PullRequest {{
-                            {self._get_labels_query_block(enable_paging=True)}
+                            {self._get_labels_query_block(page_size=100, enable_paging=True)}
                         }}
                     }}
                 }}
@@ -1460,6 +1538,7 @@ class GithubClient:
 
     def get_users_count(self, login: str) -> int:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}"){{
                     users: membersWithRole {{
                         totalCount
@@ -1478,6 +1557,7 @@ class GithubClient:
         priv_filter = '(privacy: PRIVATE)' if only_private else ''
 
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}"){{
                     repos: repositories{priv_filter} {{
                         totalCount
@@ -1496,6 +1576,7 @@ class GithubClient:
         self, login: str, page_size: int = 10
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
             organization(login: "{login}") {{
                     repositories(first: {page_size}, after: %s) {{
                         pageInfo {{
@@ -1542,6 +1623,7 @@ class GithubClient:
         self, login: str, repo_name: str, page_size=100
     ) -> Generator[dict, None, None]:
         query_body = f"""{{
+                {GQL_RATELIMIT_INFO_BLOCK}
                 organization(login: "{login}") {{
                         repository(name: "{repo_name}") {{
                             name
@@ -1571,3 +1653,54 @@ class GithubClient:
         ):
             for pr in result['data']['organization']['repository']['prs_query']['prs']:
                 yield pr
+
+    def get_pr_comments_and_reactions(
+        self, login: str, repo_name: str, pr_numbers: list[int]
+    ) -> Generator[tuple[int, list[dict]], None, None]:
+        """Hyper specific helper function for getting the historic reactions on PR comments.
+        This is done in a very specific way to minimize the number of API calls required to get
+        this data, as it is a lot of data to pull.
+
+        Args:
+            login (str): The Organization Login
+            repo_name (str): The Repo Name
+            pr_numbers (list[int]): A set of PRs to fetch for
+
+        Yields:
+            Generator[dict[int, list[dict]], None, None]: Yields a PR Number a list of associated comments
+        """
+
+        def _generate_pr_subquery(pr_number: int) -> str:
+            return f"""
+                pr_{pr_number}: pullRequest(number: {pr_number}) {{
+                    {self._get_pr_reviews_query_block(page_size=25, comment_page_size=10, reactions_page_size=5, enable_paging=False)}
+                }}
+            """
+
+        for pr_number_batch in batch_iterable(pr_numbers, 25):
+            subqueries = '\n'.join(_generate_pr_subquery(n) for n in pr_number_batch)
+            query_body = f"""
+                {{
+                {GQL_RATELIMIT_INFO_BLOCK}
+                organization(login: "{login}") {{
+                    repo: repository(name: "{repo_name}") {{
+                        {subqueries}
+                    }}
+                }}
+            }}
+            """
+            result = self.get_raw_result(query_body=query_body)
+            for pr_number in pr_number_batch:
+                api_pr = result['data']['organization']['repo'].get(f'pr_{pr_number}')
+                if not api_pr:
+                    logger.warning(f'No data found for PR {pr_number} in repo {repo_name}')
+                    continue
+
+                reviews = (
+                    [r for r in self.get_pr_reviews(login, repo_name, pr_number=pr_number)]
+                    if self._need_to_fetch_with_deeper_reviews_query(api_pr)
+                    else api_pr['reviewsQuery']['reviews']
+                )
+
+                comments = self.get_pr_comments_from_reviews(reviews)
+                yield pr_number, comments
