@@ -1,6 +1,7 @@
+import re
 from dataclasses import dataclass, field
 from datetime import datetime
-from enum import IntEnum
+from enum import Enum, IntEnum
 from functools import wraps
 from typing import Any, Dict, List, Optional
 
@@ -137,11 +138,37 @@ class StandardizedPullRequestComment(StandardizedObject):
     body: str
     created_at: datetime
     system_generated: Optional[bool] = None
+    reactions: list = field(default_factory=list)  # Currently only supported by Github
 
     def strip_text_content(self):
         self.body = sanitize_text(self.body)
 
 
+@dataclass
+class StandardizedPullRequestCommentReaction(StandardizedObject):
+    user: Optional[StandardizedUser]
+    emoji_code: str
+    created_at: datetime
+
+    UNICODE_REGEX = r'U\+[0-9A-F]+'
+    UNKNOWN_EMOJI_REGEX = r'UNKNOWN \([\w,-]+\)'
+    ACCEPTABLE_EMOJI_CODE_REGEX = f'({UNICODE_REGEX})|({UNKNOWN_EMOJI_REGEX})'
+
+    @staticmethod
+    def emoji_code_is_valid(emoji_code: str) -> bool:
+        return (
+            re.match(StandardizedPullRequestCommentReaction.UNICODE_REGEX, emoji_code) is not None
+        )
+
+    def __post_init__(self):
+        if not re.match(self.ACCEPTABLE_EMOJI_CODE_REGEX, self.emoji_code):
+            raise ValueError(
+                f"Invalid emoji code: {self.emoji_code}. "
+                f"Emoji code must match the pattern {self.UNICODE_REGEX} (Unicode format, e.g., U+1F44D for thumbs up) "
+                "or UNKNOWN (for unrecognized reactions)."
+            )
+
+
 # TODO: This exists in the Jellyfish source code
 # and has been copied over. The source code should be
 # replaced with this, so we have one "source of truth".
