@@ -23,6 +23,7 @@ from jf_ingest.jf_jira.auth import get_jira_connection
 from jf_ingest.jf_jira.downloaders import (
     download_boards_and_sprints,
     download_fields,
+    download_global_components,
     download_issuelinktypes,
     download_issuetypes,
     download_priorities,
@@ -67,6 +68,7 @@ def _generate_jira_event(event_name: str) -> JiraIngestEvent:
 class JiraObject(Enum):
     JiraFields = "jira_fields"
     JiraProjectsAndVersions = "jira_projects_and_versions"
+    JiraGlobalComponents = "jira_global_components"
     JiraUsers = "jira_users"
     JiraResolutions = "jira_resolutions"
     JiraIssueTypes = "jira_issuetypes"
@@ -590,6 +592,24 @@ def _run_load_and_push_to_s3(ingest_config: IngestionConfig) -> bool:
         save_locally=ingest_config.save_locally,
         upload_to_s3=ingest_config.upload_to_s3,
     )
+
+    #######################################################################
+    # Optionally Fetch Global Components
+    #######################################################################
+    if jira_config.download_global_components:
+        global_components = download_global_components(
+            jira_connection=jira_connect_or_fallback_connection,
+        )
+        ingest_io_helper.write_json_to_local_or_s3(
+            object_name=JiraObject.JiraGlobalComponents.value,
+            subdirectory=SubDirectory.JIRA,
+            json_data=global_components,
+            save_locally=ingest_config.save_locally,
+            upload_to_s3=ingest_config.upload_to_s3,
+        )
+    else:
+        logger.info('Skipping download of global components')
+
     #######################################################################
     # Jira Fields
     # NOTE: Here we need to download the fields from the fields endpoint
