@@ -272,6 +272,7 @@ def make_hash(o: Union[set, list, tuple, dict, Any]) -> Union[int, tuple, None]:
 @jelly_trace
 def identify_custom_field_mismatches(
     ingest_config: IngestionConfig,
+    use_jql_enhanced_search: bool = False,
     nthreads: int = 10,
     max_issues_to_process: Optional[int] = None,
     mark_for_redownload: Optional[int] = None,
@@ -289,6 +290,7 @@ def identify_custom_field_mismatches(
         fields, save that into an "updates" NamedTuple, which we can POST to Jellyfish to update the data.
 
     :param ingest_config: IngestionConfig object
+    :param use_jql_enhanced_search: Whether to use JQL Enhanced Search API (/search/jql) instead of legacy API (/search)
     :param nthreads: Number of threads to use for downloading issues from Jira
     :param max_issues_to_process: Maximum number of issues to process. If None, all issues will be processed.
     :param mark_for_redownload: Send jira issue ids to mark them for redownload in Jellyfish.
@@ -320,7 +322,9 @@ def identify_custom_field_mismatches(
     # is fine here and is faster.
     if not (jira_config := ingest_config.jira_config):
         raise Exception(f'Ingest config was provided without a Jira Configuration!')
-    jira_connection = get_jira_connection(config=jira_config)
+    jira_connection = get_jira_connection(
+        config=jira_config, use_jql_enhanced_search=use_jql_enhanced_search
+    )
 
     # We store the custom field values that are missing from the DB, missing from Jira,
     # and out of sync in separate lists - this allows for easier processing on the
@@ -367,7 +371,9 @@ def identify_custom_field_mismatches(
         # it's possible we'll be able to use a larger batch size if querying a subset
         # of fields.
         batch_size = get_jira_search_batch_size(
-            jira_connection, fields=['key'] + list(field_map.keys())
+            jira_connection,
+            fields=['key'] + list(field_map.keys()),
+            use_jql_enhanced_search=use_jql_enhanced_search,
         )
 
         # Note: this is parallelized and returns a generator, so we're
@@ -386,6 +392,7 @@ def identify_custom_field_mismatches(
                 include_fields=include_fields,
                 hide_tqdm=True,
                 adaptive_throttler=jira_request_throttler,
+                use_jql_enhanced_search=use_jql_enhanced_search,
             )
 
         # Now we need to compare the custom field values from the DB with the actual issue data from Jira.
