@@ -11,7 +11,7 @@ import time
 import traceback
 from collections import namedtuple
 from concurrent.futures import FIRST_COMPLETED, Future, ThreadPoolExecutor, wait
-from datetime import datetime
+from datetime import datetime, timezone
 from enum import Enum
 from types import TracebackType
 from typing import Any, Callable, Generator, Iterable, Optional, Type, cast
@@ -969,3 +969,18 @@ def get_id_from_gid(gitlab_gid: str, object_name: str) -> str:
     Gitlab Format: gid://gitlab/{object_name}/{ID_NUMBER}
     """
     return gitlab_gid.split(f'gid://gitlab/{object_name}/')[1]
+
+
+def normalize_datetime(dt: Any, tz: timezone) -> Any:
+    if type(dt) is datetime:
+        if dt.year == 1:
+            # HACK: If year is 1 than this is the default datetime.min value,
+            # so it's not going to make a difference if we adjust it or not.
+            # If you do attempt to adjust it you will get an overflow error.
+            # To avoid all this and to keep things simple, don't bother adjusting it.
+            return dt
+        return dt.astimezone(tz)
+    else:
+        # Sometimes objects are passed in as Dates and not Datetimes,
+        # so we can't adjust the timezone on them
+        return dt
