@@ -3,7 +3,7 @@ import logging
 import math
 import sys
 from ast import parse
-from datetime import datetime
+from datetime import datetime, timedelta, timezone
 from unittest.mock import patch
 
 import pytest
@@ -23,6 +23,7 @@ from jf_ingest.utils import (
     hash_filename,
     parse_gitlab_api_version,
     retry_for_status,
+    normalize_datetime
 )
 
 logger = logging.getLogger(__name__)
@@ -520,3 +521,27 @@ def test_parse_gitlab_api_version():
     versions_to_test = ['17.4.0', '17.4.0-pre', '17.4.0-ee', '17-stuff.4-random.0-things']
     for version_to_test in versions_to_test:
         assert parse_gitlab_api_version(version_to_test) == '17.4.0'
+
+def test_normalize_timezone():
+    UTC = timezone.utc
+    los_angeles = timezone(timedelta(hours=-8))
+    new_york = timezone(timedelta(hours=-5))
+    
+    base_time = datetime(2024, 1, 1, 18, 0, 0, tzinfo=UTC)
+    
+    assert normalize_datetime(base_time, UTC) == base_time
+    assert normalize_datetime(base_time, los_angeles) == datetime(2024, 1, 1, 10, 0, 0, tzinfo=los_angeles)
+    assert normalize_datetime(base_time, new_york) == datetime(2024, 1, 1, 13, 0, 0, tzinfo=new_york)
+    
+    base_time = datetime(2024, 1, 1, 0, 0, 0, tzinfo=UTC)  # Test date change
+    
+    assert normalize_datetime(base_time, UTC) == base_time
+    assert normalize_datetime(base_time, los_angeles) == datetime(2023, 12, 31, 16, 0, 0, tzinfo=los_angeles)
+    assert normalize_datetime(base_time, new_york) == datetime(2023, 12, 31, 19, 0, 0, tzinfo=new_york)
+    
+    base_time = datetime(1, 1, 1, 1, 1, 1, tzinfo=UTC)  # Test overflow error. If the date is low enough, we don't bother messing with timezone
+    
+    assert normalize_datetime(base_time, UTC) == base_time
+    assert normalize_datetime(base_time, new_york) == base_time
+    assert normalize_datetime(base_time, los_angeles) == base_time
+    
\ No newline at end of file
